package com.diagbot.facade;

import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.entity.node.BillItem;
import com.diagbot.entity.node.BillConcept;
import com.diagbot.entity.node.BillCondition;
import com.diagbot.repository.BillManRepository;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.ConceptIndexVO;
import com.diagbot.vo.ConditionIndexVO;
import com.diagbot.vo.BillPageVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/11/17 13:53
 */
@Component
public class BillManFacade {
    @Autowired
    BillManRepository billManRepository;
    @Autowired
    DictionaryFacade dictionaryFacade;

    /**
     * 获取开单合理项列表
     *
     * @param billPageVO
     * @return
     */
    public Page<BillItem> getPage(BillPageVO billPageVO) {
        List<DictionaryInfoDTO> dicTypeConvert = dictionaryFacade.getListByGroupType(8);
        List<DictionaryInfoDTO> dicIndicationManType = dictionaryFacade.getListByGroupType(11);
        Map<String, String> dicTypeConvertMap
                = EntityUtil.makeMapWithKeyValue(dicTypeConvert, "name", "val");
        Map<String, String> dicIndicationManTypeMap
                = EntityUtil.makeMapWithKeyValue(dicIndicationManType, "val", "name");
        List<String> labels = Lists.newArrayList();
        if (ListUtil.isNotEmpty(billPageVO.getTypes())) {
            for (Integer type : billPageVO.getTypes()) {
                if (dicIndicationManTypeMap.containsKey(type.toString())
                        && dicTypeConvertMap.containsKey(dicIndicationManTypeMap.get(type.toString()))) {
                    labels.add(dicTypeConvertMap.get(dicIndicationManTypeMap.get(type.toString())));
                }
            }
        }
        billPageVO.setLabels(labels);
        Pageable pageable = PageRequest.of(billPageVO.getNumber(), billPageVO.getSize());
        Page<BillItem> page = billManRepository.getPage(billPageVO.getLabels(),
                billPageVO.getConceptName(),
                billPageVO.getConditionName(),
                billPageVO.getRelationStatus(),
                pageable);
        return page;
    }

    /**
     * 条件明细检索
     *
     * @param conditionIndexVO
     * @return
     */
    public List<BillCondition> conditionIndex(ConditionIndexVO conditionIndexVO) {
        List<DictionaryInfoDTO> dicTypeConvert = dictionaryFacade.getListByGroupType(8);
        List<DictionaryInfoDTO> dicIndicationManType = dictionaryFacade.getListByGroupType(11);
        Map<String, String> dicTypeConvertMap
                = EntityUtil.makeMapWithKeyValue(dicTypeConvert, "name", "val");
        Map<String, String> dicIndicationManTypeMap
                = EntityUtil.makeMapWithKeyValue(dicIndicationManType, "val", "name");
        if (dicIndicationManTypeMap.containsKey(conditionIndexVO.getType().toString())
                && dicTypeConvertMap.containsKey(dicIndicationManTypeMap.get(conditionIndexVO.getType().toString()))) {
            conditionIndexVO.setConceptLabel(dicTypeConvertMap.get(dicIndicationManTypeMap.get(conditionIndexVO.getType().toString())));
        }
        List<String> relationTypes = billManRepository.getRelationTypes(conditionIndexVO.getConceptLabel());
        List<String> conditionLabels = Lists.newArrayList();
        //条件明细筛选范围
        if (ListUtil.isNotEmpty(relationTypes)) {
            for (String relationType : relationTypes) {
                int index = relationType.indexOf("禁忌");
                if (index > 0) {
                    conditionLabels.add(relationType.substring(index + 2));
                }
            }
        }
        //排除项
        List<BillCondition> notInConditions
                = billManRepository.getNotInConditions(conditionIndexVO.getConceptName(), conditionIndexVO.getConceptLabel());
        List<Long> notInIds = Lists.newArrayList();
        if (ListUtil.isNotEmpty(notInConditions) && conditionIndexVO.getRuleType().equals("文本类型")) {
            notInIds = notInConditions.stream().map(i -> i.getId()).distinct().collect(Collectors.toList());
        }
        List<BillCondition> conditions = billManRepository.conditionIndex(conditionLabels,
                notInIds,
                conditionIndexVO.getConditionName(),
                conditionIndexVO.getConditionName(),
                100);

        return conditions;
    }

    /**
     * 开单项检索
     *
     * @param conceptIndexVO
     * @return
     */
    public List<BillConcept> conceptIndex(ConceptIndexVO conceptIndexVO) {
        List<String> conceptLabels = Lists.newArrayList();
        List<DictionaryInfoDTO> dicTypeConvert = dictionaryFacade.getListByGroupType(8);
        List<DictionaryInfoDTO> dicIndicationManType = dictionaryFacade.getListByGroupType(11);
        Map<String, String> dicTypeConvertMap
                = EntityUtil.makeMapWithKeyValue(dicTypeConvert, "name", "val");
        Map<String, String> dicIndicationManTypeMap
                = EntityUtil.makeMapWithKeyValue(dicIndicationManType, "val", "name");
        if (dicIndicationManTypeMap.containsKey(conceptIndexVO.getType().toString())
                && dicTypeConvertMap.containsKey(dicIndicationManTypeMap.get(conceptIndexVO.getType().toString()))) {
            conceptIndexVO.setConceptLabel(dicTypeConvertMap.get(dicIndicationManTypeMap.get(conceptIndexVO.getType().toString())));
            conceptLabels.add(conceptIndexVO.getConceptLabel());
        }
        List<BillConcept> concepts = billManRepository.conceptIndex(conceptLabels,
                conceptIndexVO.getConceptName(),
                conceptIndexVO.getConceptName(),
                100);

        return concepts;
    }
}