package com.diagbot.repository;

import com.diagbot.entity.node.BillConcept;
import com.diagbot.entity.node.BillCondition;
import com.diagbot.entity.node.BillItem;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.neo4j.annotation.Query;
import org.springframework.data.neo4j.repository.Neo4jRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/11/17 13:26
 */
public interface BillManRepository extends Neo4jRepository<BillItem,Long> {

    @Query(value = "match(m)-[r]->(n) \n" +
            "where 1 = 1 \n" +
            "and (case {labels} is not null and size({labels})>0 when true then labels(m)[0] in {labels} else 1 = 1 end ) \n" +
            "and (case {conceptName} is not null and {conceptName} <> '' when true then m.name contains {conceptName} else 1 = 1 end ) \n" +
            "and (case {conditionName} is not null and {conditionName} <> '' when true then n.name contains {conditionName} else 1 = 1 end ) \n" +
            "and (case {relationStatus} is not null and {relationStatus} <> '' when true then r.状态 = {relationStatus} else 1 = 1 end ) \n" +
            "and type(r) contains('禁忌') \n" +
            "and m.状态=1 \n" +
            "and n.状态=1 \n" +
            "return n.name + '开单' + m.name as ruleName, r.规则类型 as ruleType,\n" +
            "case labels(m)[0] " +
            "when '实验室检查套餐名' then 1 " +
            "when '辅助检查名称' then 2 " +
            "when '辅助检查子项目名称' then 3 " +
            "when '医保手术和操作名称' then 4 " +
            "when '药品注册名称' then 5 " +
            "end as type," +
            "id(m) as conceptId,m.name as conceptName,labels(m)[0] as conceptType,\n" +
            "id(r) as relationId,type(r) as relationName,r.状态 as relationStatus, \n" +
            "id(n) as conditionId,n.name as conditionName,labels(n)[0] as conditionType  \n" +
            "order by id(r) desc",
            countQuery = "match(m)-[r]->(n) \n" +
                    "where 1 = 1 \n" +
                    "and (case {labels} is not null and size({labels})>0 when true then labels(m)[0] in {labels} else 1 = 1 end ) \n" +
                    "and (case {conceptName} is not null and {conceptName} <> '' when true then m.name contains {conceptName} else 1 = 1 end ) \n" +
                    "and (case {conditionName} is not null and {conditionName} <> '' when true then n.name contains {conditionName} else 1 = 1 end ) \n" +
                    "and (case {relationStatus} is not null and {relationStatus} <> '' when true then r.状态 = {relationStatus} else 1 = 1 end ) \n" +
                    "and type(r) contains('禁忌') \n" +
                    "and m.状态=1 \n" +
                    "and n.状态=1 \n" +
                    "return count(n)")
    Page<BillItem> getPage(@Param("labels") List<String> labels,
                           @Param("conceptName") String conceptName,
                           @Param("conditionName") String conditionName,
                           @Param("relationStatus") String relationStatus,
                           Pageable pageable);

    @Query("match (m)-[r]->(n) " +
            "where type(r) contains '禁忌' " +
            "and labels(m)[0] = $conceptLabel " +
            "return distinct type(r)")
    List<String> getRelationTypes(@Param("conceptLabel") String conceptLabel);

    @Query("MATCH (n) \n" +
            "WHERE (toLower(n.`name`) = toLower($name) OR toLower(n.`拼音编码`) = toLower($pycode))  \n" +
            "AND labels(n)[0] in {labels}  \n" +
            "AND (case {notInIds} is not null and size({notInIds})>0 when true then not(id(n) in {notInIds}) else 1 = 1 end ) \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel \n" +
            "union \n" +
            "MATCH (n) \n " +
            "WHERE (toLower(n.`name`) starts with toLower($name) OR toLower(n.`拼音编码`) starts with toLower($pycode)) \n " +
            "AND labels(n)[0] in {labels}  \n" +
            "AND (case {notInIds} is not null and size({notInIds})>0 when true then not(id(n) in {notInIds}) else 1 = 1 end ) \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel  \n" +
            "union \n" +
            "MATCH (n) \n" +
            "WHERE (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode))  \n" +
            "AND labels(n)[0] in {labels}  \n" +
            "AND (case {notInIds} is not null and size({notInIds})>0 when true then not(id(n) in {notInIds}) else 1 = 1 end ) \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel \n")
    List<BillCondition> conditionIndex(@Param("labels") List<String> labels,
                                       @Param("notInIds") List<Long> notInIds,
                                       @Param("name") String name,
                                       @Param("pycode") String pycode,
                                       @Param("size") Integer size);

    @Query("MATCH(m)-[r]->(n) \n" +
            "where m.name={conceptName} \n" +
            "and {conceptLabel} in labels(m) \n" +
            "and type(r) contains '禁忌' \n" +
            "return ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conditionLabel \n")
    List<BillCondition> getNotInConditions(@Param("conceptName") String conceptName, @Param("conceptLabel") String conceptLabel);

    @Query("MATCH (n) \n" +
            "WHERE (toLower(n.`name`) = toLower($name) OR toLower(n.`拼音编码`) = toLower($pycode))  \n" +
            "AND labels(n)[0] in {labels}  \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conceptLabel \n" +
            "union \n" +
            "MATCH (n) \n " +
            "WHERE (toLower(n.`name`) starts with toLower($name) OR toLower(n.`拼音编码`) starts with toLower($pycode)) \n " +
            "AND labels(n)[0] in {labels}  \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conceptLabel  \n" +
            "union \n" +
            "MATCH (n) \n" +
            "WHERE (toLower(n.`name`) CONTAINS toLower($name) OR toLower(n.`拼音编码`) CONTAINS toLower($pycode))  \n" +
            "AND labels(n)[0] in {labels}  \n" +
            "WITH n SKIP 0 LIMIT $size \n" +
            "RETURN ID(n) as id, n.name as name,n.状态 as status,labels(n)[0] as conceptLabel \n")
    List<BillConcept> conceptIndex(@Param("labels") List<String> labels,
                                   @Param("name") String name,
                                   @Param("pycode") String pycode,
                                   @Param("size") Integer size);
}